'use strict';

/*
 3 tipi di sensore:
 type: 0 - comm + temp            = sensore temperatura
 type: 1 - comm + temp + hum      = sensore umidità
 type: 2 - comm + temp + in + out = sensore i/o
 type: 3 - AD converter (non in uso)
 */

var Sensors = function(){
    var vars = {
            cfg: {},
            $installTab : $(".js-sensors-install-tab"),
            $configTab : $(".js-sensors-config-tab"),
            sensorNames : [
                {type: 0, name: "Temperature Sensor"},
                {type: 1, name: "Humidity Sensor"},
                {type: 2, name: "I/O Sensor"},
                {type: 3, name: "AD Converter"}
            ],
            dbg: true
        },
        config = {
            hideClass : 'hidden'
        },

    // ---- ajax calls

        fetchConfiguration = function fetchConfiguration (){
            return $.ajax({
                url:'cgi-bin/config.cgi',
                method: 'get',
                dataType : 'json'
            })
        },

        resetSensors = function resetSensors (){
            return $.ajax({
                url:'cgi-bin/new_conf_sensor.cgi',
                method: 'get',
                dataType : 'json'
            })
        },

        sensorsListener = function sensorsListener(){
            return $.ajax({
                url:'cgi-bin/add_sensor.cgi',
                method: 'get',
                dataType : 'json'
            });
        },

        detectSensors = function detectSensors(){
            return $.ajax({
                url:'cgi-bin/detected_sensors.cgi',
                method: 'get',
                dataType : 'json'
            });
        },

        finishSensors = function finishSensors(){
            return $.ajax({
                url:'cgi-bin/finish_sensor.cgi',
                method: 'post',
                dataType : 'json'
            });
        },

    // ----

        getSensorName = function(type){
            return _.pluck(_.filter( vars.sensorNames, { 'type': type } ), 'name')[0];
        },

        resetCheck = function  resetCheck(){
            var resetCheckPromise = $.Deferred();

            // if there is at least 1 sensor
            if(vars.cfg.sensor0_type !== undefined){
                // proceed without resetting (resolve, but with a "false" value)
                resetCheckPromise.resolve(false);
            }else{
                // if there aren't sensors,
                // start a new configuration by calling new_conf cgi
                resetSensors()
                    .done(function(res){
                        // if the response was OK, then resolve with a "true" value
                        if(res.response === "ok"){
                            //proceed with reset
                            resetCheckPromise.resolve(true);
                        }else{
                            // otherwise reject (error with token)
                            resetCheckPromise.reject(res);
                        }
                    })
                    .error(function(res){
                        // ABORTING FOR ERROR (server error)
                        resetCheckPromise.reject(res);
                    });
            }
            return resetCheckPromise.promise();
        },

        startListening = function startListening(){
            var $steps = $("#sensors-install").find(".step");
            var $stepResult = $("#listening-result");
            var $listeningStep = $("#listening-step");

            $steps.addClass(config.hideClass);
            $listeningStep.removeClass(config.hideClass);

            // reset before listening ONLY if no sensors are installed (initialization)
            resetCheck()
                .then(
                    function resolve(res){

                        var sensorsListenerXHR = sensorsListener();

                        sensorsListenerXHR
                            .done(function(res){
                                if(res.response === "ok"){
                                    //parse result and update $stepResult
                                    var detectSensorsXHR = detectSensors();

                                    detectSensorsXHR.done(function(detectSensorsRes){

                                        if(detectSensorsRes.response === "ok"){
                                            var tmpSensors = detectSensorsRes.data;
                                            var i;
                                            var $foundLabel = $stepResult.find(".main-title")
                                            var $sensorsList = $stepResult.find(".js-found-sensors").empty();

                                            if(tmpSensors.length > 0){
                                                // 1 or more sensors found
                                                $foundLabel
                                                    .text("The system has found ");
                                                $("<strong>")
                                                    .text("this " + tmpSensors.length + " sensors")
                                                    .appendTo($foundLabel);

                                                for(i=0; i < tmpSensors.length; i++ ){
                                                    $('<strong>')
                                                        .addClass("sensor-name")
                                                        .text(getSensorName(tmpSensors[i].type))
                                                        .appendTo($sensorsList);
                                                }
                                            }else{
                                                //no sensors found

                                                $foundLabel
                                                    .text("The system didn't find ");
                                                $("<strong>")
                                                    .text("any sensor")
                                                    .appendTo($foundLabel);

                                                //TODO: change call to actions to "listen again" or similar
                                            }
                                            //show result step
                                            $steps.addClass(config.hideClass);
                                            $stepResult.removeClass(config.hideClass);

                                        }else{
                                            showError(detectSensorsRes.message);
                                        }
                                    });
                                    detectSensorsXHR.error(function(err){
                                        showError(err.statusText);

                                        //show result step
                                        $steps.addClass(config.hideClass);
                                        $stepResult.removeClass(config.hideClass);
                                    });
                                }else{
                                    setError(res.message);
                                }
                            });
                        sensorsListenerXHR.error(function(err){
                            showError(err.statusText);
                        });
                    },
                    function reject(res){
                        showError(Config.get('btnSaveErrorMessage'));
                        initSensorInstallation();
                    }
                );

        },

        initSensorsConfig = function initSensorsConfig(cfg){

            var $widgetWrapper = $(".js-sensors-list"),
                widgetTpls = {
                    'io' : [
                        "<div class='row'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <div class='row'>",
                        "       <div class='col-xs-6'>",
                        "           <h3 class='sub-heading'>Temperature & Digital I/O </h3>",
                        "       </div>",
                        "       <div class='col-xs-6'>",
                        "            <h3>Index: <span class='index'></span></h3>",
                        "       </div>",
                        "    </div>",
                        "       <strong class='value-info'>",
                        "       <div class='row'>",
                        "           <div class='col-xs-6'>",
                        "               <small class='icoChar temperature'> id: <span class='temp'></span></small>",
                        "           </div>",
                        "           <div class='col-xs-6'>",
                        "               <small class='icoChar input'> id: <span class='in'></span></small>",
                        "           </div>",
                        "       </div>",
                        "       </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'humidity' : [
                        "<div class='row'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <div class='row'>",
                        "       <div class='col-xs-6'>",
                        "           <h3 class='sub-heading'>Temperature & Humidity</h3>",
                        "       </div>",
                        "       <div class='col-xs-6'>",
                        "            <h3>Index: <span class='index'></span></h3>",
                        "       </div>",
                        "    </div>",
                        "    <strong class='value-info'>",
                        "      <div class='row'>",
                        "        <div class='col-xs-6'>",
                        "          <small class='icoChar humidity'> id: <span class='temp'></span></small>",
                        "        </div>",
                        "      </div>",
                        "    </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'temperature': [
                        "<div class='row'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <div class='row'>",
                        "       <div class='col-xs-6'>",
                        "            <h3 class='sub-heading'>Temperature</h3>",
                        "       </div>",
                        "       <div class='col-xs-6'>",
                        "            <h3>Index: <span class='index'></span></h3>",
                        "       </div>",
                        "    </div>",
                        "    <strong class='value-info'>",
                        "      <div class='row'>",
                        "        <div class='col-xs-6'>",
                        "          <small class='icoChar temperature'> id: <span class='temp'></span></small>",
                        "        </div>",
                        "      </div>",
                        "    </strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'unknown': [
                        "<div class='row'>",
                        "  <div class='widget-box widget-box--sensor'>",
                        "    <h3 class='sub-heading'>ERROR: unknown sensor</h3>",
                        "    <strong class='sub-title'>No data available for this sensor.</strong>",
                        "  </div>",
                        "</div>"
                    ].join(" "),

                    'no-sensor': [
                        "<div class='row'>",
                        "  The system didn't find any sensor. <a href='configuration-sensors.html'>Do you want to configure sensors?</a>",
                        "</div>"
                    ].join(" ")
                },
                i,
                tpl,
                sensorTypes = [
                    cfg.sensor0_type,
                    cfg.sensor1_type,
                    cfg.sensor2_type,
                    cfg.sensor3_type,
                    cfg.sensor4_type,
                    cfg.sensor5_type
                ],
                sensorTypesid0 = [
                    cfg.sensor0_id0,
                    cfg.sensor1_id0,
                    cfg.sensor2_id0,
                    cfg.sensor3_id0,
                    cfg.sensor4_id0,
                    cfg.sensor5_id0,
                ],
                sensorTypesid1 = [
                    cfg.sensor0_id1,
                    cfg.sensor1_id1,
                    cfg.sensor2_id1,
                    cfg.sensor3_id1,
                    cfg.sensor4_id1,
                    cfg.sensor5_id1,
                ];

            sensorTypes = _.compact(sensorTypes);
            $widgetWrapper.empty();

            if(sensorTypes && sensorTypes.length > 0){
                for(i=0; i<sensorTypes.length; i++){

                    var sensorType;
                    switch (sensorTypes[i]) {
                        case "0":
                            sensorType = 'temperature';
                            break;
                        case "1":
                            sensorType = 'humidity';
                            break;
                        case "2":
                            sensorType = 'io';
                            break;
                        default:
                            sensorType = 'unknown';
                            break;
                    }

                    tpl = $(widgetTpls[sensorType]).clone();

                    switch(sensorType){
                            case 'io':
                                tpl.find('span.in').text( sensorTypesid1[i] );
                                tpl.find('span.temp').text( sensorTypesid0[i] );
                                tpl.find('span.index').text( i );
                                break;
                            case 'humidity':
                                tpl.find('span.temp').text( sensorTypesid0[i] );
                                tpl.find('span.index').text( i );
                                break;
                            case 'temperature':
                                tpl.find('span.temp').text( sensorTypesid0[i] );
                                tpl.find('span.index').text( i );
                                break;
                            default:
                                break;
                    }

                    tpl.appendTo($widgetWrapper);
                }
            }else{
                tpl = $(widgetTpls['no-sensor']).clone();
                tpl.appendTo($widgetWrapper);
            }

            // slot che disabilita la scelta file/wizard non propagando il click
            $("#sensors_enable").on("click", function(e) {
                if( $("#sensors_enable").is(":checked") )
                    $("#list_of_sensor").removeClass("hidden");
                else
                    $("#list_of_sensor").addClass("hidden");
            });
        },

        enableSensorSections = function enableSensorSections(cfg){
            vars.cfg = cfg;

            if(vars.cfg.sensors_enable === true){
                //enable add_sensor tab
                initSensorInstallation();
                // if there is at least one sensor
                if(vars.cfg.sensor0_type !== undefined){
                    //enable config tab
                    vars.$configTab.removeClass(config.hideClass);

                    var sensorTypes = [
                        cfg.sensor0_type,
                        cfg.sensor1_type,
                        cfg.sensor2_type,
                        cfg.sensor3_type,
                        cfg.sensor4_type,
                        cfg.sensor5_type
                    ];

                    // remove falsey values from array
                    sensorTypes = _.compact(sensorTypes);

                    if(sensorTypes.indexOf("2") < 0){
                        // disable Contact logic if there are no I/O sensors
                        // disable OUTPUT CONTACT section if there are no I/O sensors
                        $("#sensors_normallyclose").prop("disabled", "disabled");
                        $("#sensors_output0").prop("disabled", "disabled");
                        $("#sensors_output1").prop("disabled", "disabled");
                        $("#sensors_output2").prop("disabled", "disabled");
                        $("#sensors_output3").prop("disabled", "disabled");
                        $("#sensors_output4").prop("disabled", "disabled");
                        $("#sensors_output5").prop("disabled", "disabled");
                        $("#sensors_output6").prop("disabled", "disabled");
                        $("#sensors_output7").prop("disabled", "disabled");
                        $("#sensors_output_temp_oor").prop("disabled", "disabled");
                        $("#sensors_output_hum_oor").prop("disabled", "disabled");
                    }

                    if(sensorTypes.indexOf("1") < 0){
                        // disable humidity if there are no Hum sensors
                        $("#sensors_threshold_hh").prop("disabled", "disabled");
                        $("#sensors_threshold_lh").prop("disabled", "disabled");
                        $("#sensors_histeresis_h").prop("disabled", "disabled");
                        $("#sensors_output_hum_oor").prop("disabled", "disabled");
                    }
                    if(sensorTypes.indexOf("0") < 0 && sensorTypes.indexOf("1") < 0 && sensorTypes.indexOf("2") < 0){
                        //disable temperature if there are no Temp sensors
                        $("#sensors_threshold_ht").prop("disabled", "disabled");
                        $("#sensors_threshold_lt").prop("disabled", "disabled");
                        $("#sensors_histeresis_t").prop("disabled", "disabled");
                        $("#sensors_output_temp_oor").prop("disabled", "disabled");
                    }

                    jcf.refreshAll();

                    //populate select dropdowns in 'sensor-config' section
                    initSensorFillCombo(cfg);

                    initSensorsConfig(cfg);
                }
                //apply configuration
                applyConfiguration(cfg);
            }
            initForms();
        },

        applyConfiguration = function applyConfiguration(cfg){
            $.each(cfg, function(k,v){
                switch(k){
                    case "device_prtk":
                        v = v.substring(0,10) + "--";
                        switch (v) {
                            case "GPSER11201--":
                            case "GPSER19601--":
                            case "SENTR19601--":
                            case "SENTR11201--":
                                $("#sensor_output5_label").text("Battery Low");
                                break;
                            default:
                                $("#sensor_output5_label").text("Transfer Active");
                                break;
                        }
                        break;
                    default: break;
                }
                var $elem = $("[name="+k+"]");

                if( k === "sensors_threshold_ht") { v = v / 10;}
                if( k === "sensors_threshold_lt") { v = v / 10;}
                if( k === "sensors_histeresis_t") { v = v / 10;}

                if(!$elem.is(":disabled")){


                    setFormElement($elem, v);
                }

                jcf.refresh( $elem );
                $elem.trigger("change");
            });
            return cfg
        },

        submitSensorsHandler = function submitSensorsHandler(form){
            var $form = $(form),
                $submit = $form.find("input[type=submit]"),
                url = $form.attr("action"),
                method = $form.attr("method"),
                formData = $form.serializeArray({ checkboxesAsBools: true }),
                data = formData,
                submitTxt = $submit.val();

            data = _.chain(data)
                .indexBy('name')
                .mapValues('value')
                .value();

            if($form.hasClass('sensor-config')){
                data.sensors_threshold_ht = data.sensors_threshold_ht * 10;
                data.sensors_threshold_lt = data.sensors_threshold_lt * 10;
                data.sensors_histeresis_t = data.sensors_histeresis_t * 10;
            }

            // append token to sent data
            // (just before sending the form, so it wont be overwritten)
            data.token = Cookies.get("token");

            //btn to saving state
            $submit.val(Config.get("waitMessage")).prop("disabled", "disabled");

            var xhr = $.ajax({
                url: url,
                method: method,
                data: data
            });
            xhr.done(function(res){
                var res = JSON.parse(res);

                if(res.response === "ok"){
                    showSuccess(res.message);
                    $submit.val(Config.get("btnSaveSuccessMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-success");
                }else{
                    showError(res.message);
                    $submit.val(Config.get("btnSaveErrorMessage"))
                        .removeClass("btn-primary")
                        .addClass("btn-danger");
                }
            });
            xhr.error(function(err){
                showError(err.statusText);
                $submit.val(Config.get("btnSaveErrorMessage"))
                    .removeClass("btn-primary")
                    .addClass("btn-danger");
            });
            xhr.always(function(){
                //btn back to initial state
                window.setTimeout(function(){
                    $submit.val(submitTxt)
                        .prop("disabled", false)
                        .removeClass("btn-danger")
                        .removeClass("btn-success")
                        .addClass("btn-primary");
                }, Config.get("feedbackMessageDuration"));
            });
        },

    // ----- Init -----

        initSensorInstallation = function initSensorInstallation(){

            var $sensorsInstall = $("#sensors-install");
            var $steps = $sensorsInstall.find(".step");

            $sensorsInstall
                .off("click", ".js-listen-btn")
                .on("click", ".js-listen-btn", function(e){
                    e.preventDefault();
                    startListening();
                });

            $sensorsInstall
                .off('click', ".js-reset-btn")
                .on("click", ".js-reset-btn", function(e){
                    e.preventDefault();
                    resetSensors().then(initSensorInstallation);
                });

            $sensorsInstall
                .off("click", ".js-sensors-finish")
                .on("click", ".js-sensors-finish", function(e) {
                    e.preventDefault();

                    var $btn = $(this);
                    var oldBtnMessage = $btn.text();
                    $btn.text(Config.get("waitMessage")).addClass("disabled");

                    var finishSensorsXHR = finishSensors();

                    finishSensorsXHR.then(function(res){
                        if(res.response === "ok"){
                            // innested tabs aren't fully supported by Bootstrap's JS,
                            // we're triggering a click on the link to change the active tab
                            // $(".js-sensors-config-tab").find("a").trigger("click");
                            window.location.href = "configuration-sensors.html#tab_sensors-configuration";
                            window.location.reload();
                        }else{
                            setError(res.message);
                        }
                    });

                    finishSensorsXHR.always(function(){
                        $btn.text(oldBtnMessage).removeClass("disabled");
                    });

                });

            $sensorsInstall
                .off("click", ".js-sensors-add-another")
                .on("click", ".js-sensors-add-another", function(e) {
                    e.preventDefault();
                    //return to first step (don't start a new listening)
                    initSensorInstallation();
                });

            $steps.addClass(config.hideClass);
            vars.$installTab.removeClass(config.hideClass);
            $("#connect-step").removeClass(config.hideClass);
        },

        initSensorFillCombo = function initSensorFillCombo(cfg){
            var $sensorConf = $("#sensors-configuration"),
                $sensorsSelect = $sensorConf.find(".js-sensor-select"),
                $optionTpl = function(){ return $("<option></option>")},
                sensorTypes = [
                    cfg.sensor0_type,
                    cfg.sensor1_type,
                    cfg.sensor2_type,
                    cfg.sensor3_type,
                    cfg.sensor4_type,
                    cfg.sensor5_type
                ];

            // remove falsey values from array
            sensorTypes = _.compact(sensorTypes);

            $.each($sensorsSelect, function( i ){
                $optionTpl().attr('value','').text('None').appendTo($sensorsSelect[i]);
                $.each(sensorTypes, function( j ) {
                    if (sensorTypes[j] == "2")
                        $optionTpl().attr( 'value', j).text(j).appendTo($sensorsSelect[i]);
                });
                jcf.refresh($sensorsSelect[i]);
            } );
        },

        initForms = function initForms(){
            var $forms = $("form");

            $.each($forms, function(i, form){
                var $form = $(form);

                $form.submit(function(e){
                    e.preventDefault();
                });

                $form.validate({
                    submitHandler : submitSensorsHandler
                });
            });
        },

        init = function init() {
            //hide install and config tabs
            vars.$installTab.addClass(config.hideClass);
            vars.$configTab.addClass(config.hideClass);

            //fetch global configuration
            fetchConfiguration()
                .then(enableSensorSections);
        };

    return {
        init: init
    }
}();
